@@ -1,17 +1,18 @@
-#!/usr/bin/python2
+#!/usr/bin/python
 # sorts resources by their included name and version information
 # usage: call from the directory containing the resources or
 # call
 # sortresources.py <path_to_seach_and_sort>
 # use sortresources.py -h to get command line option help
 
+from __future__ import print_function
+
 from xml import sax
 from xml.sax import handler
-import string
 import os.path
 import os
 import sys
-import StringIO
+from io import StringIO
 
 # parse a resource xml file and fill in data property data strucure
 def parseResource(filename, data):
@@ -63,7 +64,7 @@ def parseResource(filename, data):
               path = self.path
 
               # extract data type: the base name of the DTD
-              self.data.type = string.split(sysid,"-")[0]
+              self.data.type = str.split(sysid,"-")[0]
 
               # look in current and parent directories for dtds
               while len(path) > 0:
@@ -75,7 +76,7 @@ def parseResource(filename, data):
 
               # fallback: return empty stream, result: no dtd checking is done.
               # Not horribly bad in this context
-              print "warning, could not find requested entity", sysid
+              print("warning, could not find requested entity", sysid)
               return StringIO.StringIO("")
 
     # parse: create parser...
@@ -111,15 +112,13 @@ def getCanonicalPath(path):
 
 # scan for all XML files in the directory and rename them according to the rules
 def scanDir(sourceDir, destinationDir, function):
-    def visitor( arg, dirname, names ):
-        for file in names:
+    for dirname, dirs, files in os.walk(sourceDir):
+        for file in files:
             if len(file) > 4 and file[-4:] == ".xml":
                 path = os.path.join(dirname,file)
                 newPath = getCanonicalPath(path)
                 # call the passed function
                 function(path, os.path.join(destinationDir, newPath), newPath)
-        
-    os.path.walk(sourceDir, visitor, visitor )
 
 # move file oldFile to newFile
 def Move(oldFile, newFile, canonicalPath ):
@@ -143,7 +142,7 @@ def Print(oldFile, newFile, canonicalPath ):
         return False
 
     if doPrint:
-        print "renaming", oldFile, "->", newFile
+        print("renaming", oldFile, "->", newFile)
 
 # move file oldFile to newFile, setting apache rewrite rules to keep the file
 # fetchable from its old position
@@ -165,10 +164,10 @@ def Redirect(oldFile, newFile, canonicalPath ):
             # open .htaccess file in the old directory for reading
             accessFile = open(htaccessPath)
             for line in accessFile:
-                splitLineLower = string.split(line.lower())
-                splitLine      = string.split(line)
+                splitLineLower = str.split(line.lower())
+                splitLine      = str.split(line)
                 if splitLineLower[0] == "rewriteengine":
-                    # rewriting is already mentioned; we don't have to enable it laster.
+                    # rewriting is already mentioned; we don't have to enable it later.
                     needEngine = False
 
                 # see if there already is a rewrite rule for our file
@@ -177,7 +176,7 @@ def Redirect(oldFile, newFile, canonicalPath ):
                         # nothing to do
                         return
                     else:
-                        print "Warning: There already is a different RewriteRule for", oldFile, "in place."
+                        print("Warning: There already is a different RewriteRule for", oldFile, "in place.")
         except: pass
 
         # open .htaccess file for appended writing
@@ -192,15 +191,15 @@ def Redirect(oldFile, newFile, canonicalPath ):
 
 # print usage message and exit.
 def Options( ret ):
-    print
-    print "Usage: sortresources.py [OPTIONS] <source directory> <destination directory>"
-    print "       Omitting the destination directory makes it equal to the source."
-    print "       Omitting both directories makes them the current working directory."
-    print "Options: -r add apache rewrite rules"
-    print "         -n don't do anything, just print what would be done"
-    print "         -v print non-warning status messages"
-    print "         -h print this message"
-    print
+    print()
+    print("Usage: sortresources.py [OPTIONS] <source directory> <destination directory>")
+    print("       Omitting the destination directory makes it equal to the source.")
+    print("       Omitting both directories makes them the current working directory.")
+    print("Options: -r add apache rewrite rules")
+    print("         -n don't do anything, just print what would be done")
+    print("         -v print non-warning status messages")
+    print("         -h print this message")
+    print()
     sys.exit( ret )
     
 if __name__ == "__main__":
@@ -229,7 +228,7 @@ if __name__ == "__main__":
                 if arg[1] == "v":
                     doPrint = True
                     continue
-                print "\nUnknown option:", arg
+                print("\nUnknown option:", arg)
                 Options(-1)
             else:
                 Options(-1)
@@ -239,7 +238,7 @@ if __name__ == "__main__":
                 sourceDirectory = arg
             destinationDirectory = arg
     
-    # print sourceDirectory, destinationDirectory
+    # print(sourceDirectory, destinationDirectory)
 
     # do the work
     scanDir(sourceDirectory, destinationDirectory, function)
