@@ -167,7 +167,7 @@ inline void eHalfEdge::Unlink()
 {
     if (point && this == point->edge)
     {
-        if (prev && prev->other && prev->other)
+        if (prev && prev->other)
             point->edge = prev->other;
         else if (other && other->next)
             point->edge = other->next;
@@ -361,7 +361,7 @@ eFaceScorePair se_FindBestReplacement( const eFace *old, eFaceReplacementArgumen
 
     // return invalid return if the face was already visited
     if ( arg.visited.find( old ) != arg.visited.end() )
-        return eFaceScorePair( nullEFace, -se_maxGridSize );
+        return eFaceScorePair( nullEFace, -se_maxGridSize*se_maxGridSize );
 
     // register face as visited
     arg.visited.insert( old );
@@ -376,7 +376,7 @@ eFaceScorePair se_FindBestReplacement( const eFace *old, eFaceReplacementArgumen
         // iterate it
 
         // the currently best face/insideness pair
-        std::pair< eFace*, REAL > best( nullEFace, -100000 );
+        std::pair< eFace*, REAL > best( nullEFace, -se_maxGridSize*se_maxGridSize );
         for( eReplacementStorage::const_iterator i = storage.begin(); i != storage.end(); ++i )
         {
             // the current face/insideness pair
@@ -398,8 +398,10 @@ eFaceScorePair se_FindBestReplacement( const eFace *old, eFaceReplacementArgumen
                 current = se_FindBestReplacement( face, arg );
             }
 
-            if ( current.second > best.second && current.first )
+            if ( ( current.second > best.second && current.first ) || !best.first )
+            {
                 best = current;
+            }
         }
 
         return best;
@@ -728,7 +730,7 @@ ePoint * eGrid::DrawLine(ePoint *start, const eCoord &end, eWall *w, bool change
     //tJUST_CONTROLLED_PTR< eWall > wal( w );
 
     // sanity check
-    if ( !finite( end.x ) || !finite( end.y ) )
+    if ( !std::isfinite( end.x ) || !std::isfinite( end.y ) )
         return start;
 
     Range(end.NormSquared());
