@@ -62,6 +62,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 tCONFIG_ENUM( rResolution );
 tCONFIG_ENUM( rColorDepth );
+tCONFIG_ENUM( rVSync );
 
 SDL_Surface *sr_screen=NULL; // our window
 
@@ -98,7 +99,6 @@ std::istream & operator >> ( std::istream & s, rScreenSize const & size )
 }
 */
 
-#ifndef DEBUG
 static rScreenSettings em6(ArmageTron_320_240, false, ArmageTron_ColorDepth_16, true, false);
 static rScreenSettings em5(ArmageTron_320_240, false, ArmageTron_ColorDepth_Desktop, true, false);
 static rScreenSettings em4(ArmageTron_640_480, false,ArmageTron_ColorDepth_16);
@@ -107,7 +107,6 @@ static rScreenSettings em2(ArmageTron_640_480, true, ArmageTron_ColorDepth_16, f
 static rScreenSettings em1(ArmageTron_640_480);
 
 static rScreenSettings *emergency[MAXEMERGENCY+2]={ &lastSuccess, &lastSuccess, &em1, &em2, &em3 , &em4, &em5, &em6};
-#endif
 
 #ifdef DEBUG
 rScreenSettings currentScreensetting(ArmageTron_640_480);
@@ -139,6 +138,9 @@ static tConfItem<rResolution> screenresLast("ARMAGETRON_LAST_SCREENMODE",lastSuc
 static tConfItem<rResolution> winsize("ARMAGETRON_WINDOWSIZE",currentScreensetting.windowSize.res);
 static tConfItem<rResolution> winsizeLast("ARMAGETRON_LAST_WINDOWSIZE",lastSuccess.windowSize.res);
 
+static tConfItem<rVSync> vSync("ARMAGETRON_VSYNC",currentScreensetting.vSync);
+static tConfItem<rVSync> vSyncLast("ARMAGETRON_VSYNC_LAST",lastSuccess.vSync);
+
 static tConfItem<int> screenres_w("ARMAGETRON_SCREENMODE_W",currentScreensetting.res.width);
 static tConfItem<int> screenresLast_w("ARMAGETRON_LAST_SCREENMODE_W", lastSuccess.res.width);
 
@@ -154,7 +156,7 @@ static tConfItem<int> winsize_h("ARMAGETRON_WINDOWSIZE_H",currentScreensetting.w
 static tConfItem<int> winsizeLast_h("ARMAGETRON_LAST_WINDOWSIZE_H",lastSuccess.windowSize.height);
 
 static tConfItem<bool> fs_ci("FULLSCREEN",currentScreensetting.fullscreen);
-static tConfItem<bool> fs_lci("LAST_FULLSCREEN",currentScreensetting.fullscreen);
+static tConfItem<bool> fs_lci("LAST_FULLSCREEN",lastSuccess.fullscreen);
 
 static tConfItem<rColorDepth> tc("COLORDEPTH",currentScreensetting.colorDepth);
 static tConfItem<rColorDepth> ltc("LAST_COLORDEPTH",lastSuccess.colorDepth);
@@ -335,7 +337,7 @@ int rScreenSize::Compare( rScreenSize const & other ) const
 // *******************************************************************************************
 
 rScreenSettings::rScreenSettings( rResolution r, bool fs, rColorDepth cd, bool sdl, bool ce )
-        :res(r), windowSize(r), fullscreen(fs), colorDepth(cd), zDepth( ArmageTron_ColorDepth_Desktop ), useSDL(sdl), checkErrors(true), aspect (1)
+        :res(r), windowSize(r), fullscreen(fs), colorDepth(cd), zDepth( ArmageTron_ColorDepth_Desktop ), useSDL(sdl), checkErrors(true), vSync( ArmageTron_VSync_Default ), aspect (1)
 {
     // special case for desktop resolution: window size of 640x480
     if ( r == ArmageTron_Desktop )
@@ -387,12 +389,43 @@ static int countBits(unsigned int count)
 }
 #endif
 
+// flag indicating whether directX is supposed to be used for input (defaults to false, crashes on my Win7)
+bool sr_useDirectX = false;
+static bool use_directx_back = false;
+static void sr_SetGLAttributes( int rDepth, int gDepth, int bDepth, int zDepth )
+{
+    // SDL 1.1 required
+#ifdef SDL_OPENGL
+    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, rDepth );
+    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, gDepth );
+    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, bDepth );
+    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, zDepth );
+    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+
+#if SDL_VERSION_ATLEAST(1, 2, 10)
+    // requires SDL 1.2.10
+    switch (currentScreensetting.vSync)
+    {
+    case ArmageTron_VSync_On:
+        SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 1 );
+        break;
+    case ArmageTron_VSync_Off:
+    case ArmageTron_VSync_MotionBlur:
+        SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 0 );
+        break;
+    case ArmageTron_VSync_Default:
+        break;
+    }
+#endif
+#endif
+}
+
 static bool lowlevel_sr_InitDisplay(){
 #ifndef DEDICATED
     rScreenSize & res = currentScreensetting.fullscreen ? currentScreensetting.res : currentScreensetting.windowSize;
 
     // update pixel aspect ratio
-    if ( res.res != ArmageTron_Invalid )
+    if ( res.res != ArmageTron_Invalid && size_t(res.res) < sizeof(aspect)/sizeof(aspect[0]) )
         currentScreensetting.aspect = aspect[res.res];
 
 #ifndef DIRTY
@@ -402,8 +435,60 @@ static bool lowlevel_sr_InitDisplay(){
     sr_screenWidth = res.width;
     sr_screenHeight= res.height;
 
+    // desktop color depth
+    static int desktopCD_R = 5;
+    static int desktopCD_G = 5;
+    static int desktopCD_B = 5;
+    static int desktopCD   = 16;
+    // desktop resolution
+    static int sr_desktopWidth = 0, sr_desktopHeight = 0;
+
+    // determine those values
+    if ( sr_desktopWidth == 0 && !sr_screen )
+    {
+        // select sane defaults in case the following operation fails
+        sr_desktopWidth = 800;
+        sr_desktopHeight = 600;
+        
+        const SDL_VideoInfo* videoInfo     = SDL_GetVideoInfo( );
+        if( videoInfo )
+        {
+            const SDL_PixelFormat* pixelFormat = videoInfo->vfmt;
+
+            // don't accept anything less than 15 bpp, OpenGL doesn't like indexed colors.
+            if( pixelFormat && 15 <= pixelFormat->BitsPerPixel && NULL == pixelFormat->palette )
+            {
+                desktopCD    = pixelFormat->BitsPerPixel;
+                desktopCD_R  = countBits(pixelFormat->Rmask);
+                desktopCD_G  = countBits(pixelFormat->Gmask);
+                desktopCD_B  = countBits(pixelFormat->Bmask);
+            }
+
+            // the struct components we read here only exist since 
+            // SDL 1.2.10. The version check here is to safeguard against
+            // code compiled against SDL 1.2.10, but linked with an earlier
+            // version, accessing data out of bounds.
+#if SDL_VERSION_ATLEAST(1, 2, 10)
+            if( sr_DesktopScreensizeSupported() )
+            {
+                sr_desktopWidth  = videoInfo->current_w;
+                sr_desktopHeight = videoInfo->current_h;
+            }
+#endif
+        }
+    }
+
     if (!sr_screen)
     {
+        {
+            tOutput o("$game_name");
+            tString s;
+            s << o;
+            SDL_WM_SetCaption(s, s);
+        }
+
+        SDL_EnableUNICODE(1);
+
         int singleCD_R	= 5;
         int singleCD_G	= 5;
         int singleCD_B	= 5;
@@ -417,12 +502,10 @@ static bool lowlevel_sr_InitDisplay(){
             break;
         case ArmageTron_ColorDepth_Desktop:
             {
-                const SDL_VideoInfo* videoInfo     = SDL_GetVideoInfo( );
-                const SDL_PixelFormat* pixelFormat = videoInfo->vfmt;
-                fullCD         		               = pixelFormat->BitsPerPixel;
-                singleCD_R                         = countBits(pixelFormat->Rmask);
-                singleCD_G                         = countBits(pixelFormat->Gmask);
-                singleCD_B                         = countBits(pixelFormat->Bmask);
+                fullCD     = desktopCD;
+                singleCD_R = desktopCD_R;
+                singleCD_G = desktopCD_G;
+                singleCD_B = desktopCD_B;
             }
             break;
         case ArmageTron_ColorDepth_32:
@@ -444,12 +527,7 @@ static bool lowlevel_sr_InitDisplay(){
 #ifdef SDL_OPENGL
         if (currentScreensetting.useSDL)
         {
-            // SDL 1.1 required
-            SDL_GL_SetAttribute( SDL_GL_RED_SIZE, singleCD_R );
-            SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, singleCD_G );
-            SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, singleCD_B );
-            SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, zDepth );
-            SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+            sr_SetGLAttributes( singleCD_R, singleCD_G, singleCD_B, zDepth );
         }
 #else
         currentScreensetting.useSDL = false;
@@ -509,13 +587,13 @@ static bool lowlevel_sr_InitDisplay(){
                   attrib);
 
             // if not quite right
-            if (CD < fullCD){
+            if (CD < 15){
                 // check if the other fs/windowed mode is better
                 int CD_fsinv = SDL_VideoModeOK
                                (sr_screenWidth, sr_screenHeight,   fullCD,
                                 attrib^SDL_FULLSCREEN);
 
-                if (CD_fsinv > fullCD){
+                if (CD_fsinv >= 15){
                     // yes! change the mode
                     currentScreensetting.fullscreen=!currentScreensetting.fullscreen;
                     attrib ^= SDL_FULLSCREEN;
@@ -530,40 +608,12 @@ static bool lowlevel_sr_InitDisplay(){
 #ifdef SDL_OPENGL
                 if (currentScreensetting.useSDL)
                 {
-                    // SDL 1.1 required
-                    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
-                    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
-                    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
-                    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
-                    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+                    sr_SetGLAttributes( 5, 5, 5, 16 );
                 }
 #endif
             }
         }
 
-        // determine desktop resolution
-        static int sr_desktopWidth = 0, sr_desktopHeight = 0;
-        if ( sr_desktopWidth == 0 && !sr_screen )
-        {
-            // select sane defaults in case the following operation fails
-            sr_desktopWidth = 640;
-            sr_desktopHeight = 480;
-
-            if ( sr_DesktopScreensizeSupported()
-#ifdef DEBUG
-                 && currentScreensetting.fullscreen && sr_screenWidth + sr_screenHeight == 0
-#endif
-                )
-            {
-                sr_screen=SDL_SetVideoMode( 0, 0, CD, attrib );
-                if ( sr_screen )
-                {
-                    sr_desktopWidth = sr_screen->w;
-                    sr_desktopHeight = sr_screen->h;
-                }
-            }
-        }
-
         // if desktop resolution was selected, pick it
         if ( sr_screenWidth + sr_screenHeight == 0 )
         {
@@ -621,17 +671,30 @@ static bool lowlevel_sr_InitDisplay(){
                 */
             }
         }
+    }
 
-        // MacOSX SDL 1.2.4 crashes if we SetCaption after switch to fullscreen. (fixed in 1.2.5)
-        if(!currentScreensetting.fullscreen)
+    // sanity check texture modes
+    for(int i=rTextureGroups::TEX_GROUPS-1; i>=0; --i)
+    {
+        int & texmode = rTextureGroups::TextureMode[i];
+
+        // don't do anything for deliberately disabled textures
+        if( i == rTextureGroups::TEX_FONT || texmode >= 0 )
         {
-            tOutput o("Armagetron Advanced");
-            tString s;
-            s << o;
-            SDL_WM_SetCaption(s, s);
+            // to default if the modes have been reset for some reason
+            if( texmode == 0 )
+            {
+                texmode = default_texturemode;
+            }
+            if( texmode < GL_NEAREST )
+            {
+                texmode = GL_NEAREST;
+            }
+            if( texmode > GL_LINEAR_MIPMAP_LINEAR )
+            {
+                texmode = GL_LINEAR_MIPMAP_LINEAR;
+            }
         }
-
-        SDL_EnableUNICODE(1);
     }
 
 #ifdef DIRTY
@@ -768,6 +831,11 @@ static bool lowlevel_sr_InitDisplay(){
     sr_ResetRenderState(true);
 
     rCallbackAfterScreenModeChange::Exec();
+
+    lastSuccess=currentScreensetting;
+    failed_attempts = 0;
+    sr_useDirectX = use_directx_back;
+    st_SaveConfig();
 #endif
     return true;
 }
@@ -775,19 +843,26 @@ static bool lowlevel_sr_InitDisplay(){
 bool cycleprograminited = false;
 
 bool sr_InitDisplay(){
+    use_directx_back = sr_useDirectX;
+
     cycleprograminited = false;
     while (failed_attempts <= MAXEMERGENCY+1)
     {
-#ifndef DEBUG
         if (failed_attempts)
+        {
+#ifdef DEBUG
+            std::cout << "failed attempts:" << failed_attempts << "\n";
+            std::cout.flush();
+#endif
             currentScreensetting = *emergency[failed_attempts];
 
+            sr_useDirectX = false;
+        }
+
+        // prepare for crash, note failure and save config
         failed_attempts++;
         st_SaveConfig();
 
-        //      std::cout << failed_attempts << "\n";
-        //      std::cout.flush();
-#endif
 
 #ifdef MACOSX
         // init the screen once in windowed mode
@@ -811,7 +886,6 @@ bool sr_InitDisplay(){
         sr_LockSDL();
         if (lowlevel_sr_InitDisplay())
         {
-            lastSuccess=currentScreensetting;
             sr_UnlockSDL();
             return true;
         }
@@ -820,7 +894,6 @@ bool sr_InitDisplay(){
 
         if (lowlevel_sr_InitDisplay())
         {
-            lastSuccess=currentScreensetting;
             sr_UnlockSDL();
             return true;
         }
@@ -853,9 +926,6 @@ void sr_ExitDisplay(){
 #endif
 
     if (sr_screen){
-        failed_attempts = 0;
-        st_SaveConfig();
-
         sr_LockSDL();
         // z-man: according to man SDL_SetVideoSurface, screen should not bee freed.
         // SDL_FreeSurface(sr_screen);
@@ -885,8 +955,9 @@ bool sr_texturesTruecolor=false;
 
 bool sr_textOut=false;
 bool sr_FPSOut=true;
+bool sr_RecordingTimeOut=true;
 
-bool sr_keepWindowActive=false;
+bool sr_keepWindowActive=true;
 
 tString renderer_identification;
 
@@ -911,7 +982,7 @@ void sr_LoadDefaultConfig(){
     sr_infinityPlane=false;
     sr_lowerSky=false;
     sr_upperSky=false;
-    sr_keepWindowActive=false;
+    sr_keepWindowActive=true;
     rSysDep::swapMode_=rSysDep::rSwap_glFinish;
 
     if (software_renderer){
