@@ -754,7 +754,7 @@ public:
             {
                 hit = true;
                 distance = front.distance;
-                if (character->properties[AI_LOOP] > 3 + fabsf(winding) * 3)
+                if (character->properties[AI_LOOP] > 3 + abs(winding) * 3)
                     DetectLoop(front, frontLoop);
             }
         } while (!front.Hit() && count++ < character->properties[AI_RANGE]);
@@ -792,7 +792,7 @@ public:
 
                 if (sides[i].Hit())
                 {
-                    if (character->properties[AI_LOOP] > 6 + fabsf(winding) * 3)
+                    if (character->properties[AI_LOOP] > 6 + abs(winding) * 3)
                         DetectLoop(sides[i], sideLoop[i]);
 
                     if (sideLoop[i][1-i].loop ||
@@ -2285,9 +2285,11 @@ bool gAIPlayer::EmergencySurvive( ThinkData & data, int enemyevade, int prefered
 
     if (target && character->properties[AI_ENEMY] > 0)
     {
+        /*
         bool sdanger = false;
         for (i = DANGERLEVELS-1; i>=0; i--)
             sdanger |= sideDanger[i][0] > 4 || sideDanger[i][1] > 4;
+        */
 
         eCoord enemypos=target->Position() - Object()->Position();
         eCoord enemydir=target->Direction();
@@ -2781,8 +2783,8 @@ REAL gAIPlayer::Think(){
     // get the sensors to the left and right with the most free space
     int currentDirectionNumber = Object()->Grid()->DirectionWinding( dir );
     REAL mindistLeft = 1E+30, mindistRight = 1E+30;
-    std::auto_ptr< gAISensor > left  ( sg_GetSensor( currentDirectionNumber, *Object(), -1, side, range, corridor, mindistLeft ) );
-    std::auto_ptr< gAISensor > right ( sg_GetSensor( currentDirectionNumber, *Object(), 1, side, range, corridor, mindistRight ) );
+    std::unique_ptr< gAISensor > left  ( sg_GetSensor( currentDirectionNumber, *Object(), -1, side, range, corridor, mindistLeft ) );
+    std::unique_ptr< gAISensor > right ( sg_GetSensor( currentDirectionNumber, *Object(), 1, side, range, corridor, mindistRight ) );
 
     // count intermediate walls to the left and right as if they were in front
     {
