@@ -40,7 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gGame.h"
 #include "tRecorder.h"
 
-#include <rRender.h>
+#include "rRender.h"
 #include <math.h>
 #include "gCycle.h"
 #include <time.h>
@@ -56,12 +56,26 @@ bool subby_ShowHUD=true, subby_ShowSpeedFastest=true, subby_ShowScore=true, subb
 bool showTime=false;
 bool show24hour=false;
 REAL subby_ScoreLocX=-0.95, subby_ScoreLocY=-0.85, subby_ScoreSize =.13;
-REAL subby_FastestLocX=-0.0, subby_FastestLocY=-0.95, subby_FastestSize =.12;
+REAL subby_FastestLocX=-0.2, subby_FastestLocY=-0.95, subby_FastestSize =.13;
 REAL subby_AlivePeopleLocX=.45, subby_AlivePeopleLocY=-0.95, subby_AlivePeopleSize =.13;
 REAL subby_PingLocX=.80, subby_PingLocY=-0.95, subby_PingSize =.13;
 
 REAL max_player_speed=0;
 
+namespace
+{
+static tConfigMigration migrate([](tString const &savedInVersion){
+    // defaults for subby_FastestSize and subby_FastestLocX changed on 0.2.9.1
+    if(tConfigMigration::SavedBefore(savedInVersion, "0.2.9.1_alpha"))
+    {
+        if(subby_FastestSize < subby_AlivePeopleSize)
+            subby_FastestSize = .13; // was .12
+        if(subby_FastestLocX > -.1)
+            subby_FastestLocX = -.2; // was -0.0
+    }
+});
+}
+
 #ifndef DEDICATED
 
 void GLmeter_subby(float value,float max, float locx, float locy, float size, const char * t,bool displayvalue = true, bool reverse = false, REAL r=.5, REAL g=.5, REAL b=1)
@@ -112,8 +126,8 @@ void GLmeter_subby(float value,float max, float locx, float locy, float size, co
     max_t << "0xcccccc" << (reverse?0:max);
 
     if(displayvalue){
-        rTextField speed_t(-x*1.45*size+locx,y*1.35*size+locy,.1*size,.2*size);
-        sprintf(string,"%.1f",value);
+        rTextField speed_t(-x*1.45*size+locx,y*1.35*size+locy,.12*size,.24*size);
+        snprintf( string, sizeof( string ), "%.1f", value );
         speed_t << "0xccffff" << (string);
     }
     int length = title.Len();
@@ -124,6 +138,9 @@ void GLmeter_subby(float value,float max, float locx, float locy, float size, co
     }
 }
 
+static REAL sg_hudCacheThreshold=.0001;
+static tSettingItem< REAL > sg_minDropIntervalConf( "HUD_CACHE_THRESHOLD", sg_hudCacheThreshold );
+
 class gGLMeter
 {
 public:
@@ -139,7 +156,7 @@ public:
         REAL change = rel - oldRel_;
 
         // see if the gauge change is enough to warrant an update
-        if ( change * change * ( time - oldTime_ ) > .000001 || time < oldTime_ )
+        if ( change * change * ( time - oldTime_ ) > sg_hudCacheThreshold*sg_hudCacheThreshold || time < oldTime_ )
         {
             list_.Clear();
         }
@@ -201,7 +218,7 @@ static void tank_display_hud( ePlayerNetID* me ){
     {
         REAL newtime = tSysTimeFloat();
         REAL ts      = newtime - lastTime;
-        int newfps   = static_cast<int>(se_AverageFPS());
+        int newfps = static_cast<int>(se_AverageFPS() + .5f);
         if (fabs((newfps-fps)*ts)>4)
         {
             fps      = newfps;
@@ -424,7 +441,7 @@ static void display_hud_subby( ePlayer* player ){
                                 tColoredString message,messageColor;
                                 messageColor << "0xbf9d50";
 
-                                sprintf(fasteststring,"%.1f",max);
+                                snprintf( fasteststring, sizeof( fasteststring ), "%.1f", max );
                                 message << "  Fastest: " << name << " " << fasteststring;
                                 message.RemoveHex(); //cheers tank;
                                 int length = message.Len();
@@ -527,9 +544,9 @@ static void display_fps_subby()
     rDisplayListFiller filler( cache.list_ );
 
     float size =.15;
-    rTextField c2(.7,.85,.15*size, .3*size);
+    rTextField c2(.9-.2*rTextField::AspectWidthMultiplier(),.85,.15*size*rTextField::AspectWidthMultiplier(), .3*size);
 
-    if ( tRecorder::IsRunning() )
+    if ( sr_RecordingTimeOut && tRecorder::IsRunning() )
     {
         std::stringstream time;
         time.precision( 5 );
@@ -539,7 +556,7 @@ static void display_fps_subby()
         }
     }
 
-    int newfps   = static_cast<int>(se_AverageFPS());
+    int newfps = static_cast<int>(se_AverageFPS() + .5f);
     if (fabs((newfps-fps)*ts)>4)
     {
         fps      = newfps;
@@ -552,7 +569,7 @@ static void display_fps_subby()
     // Show the time
     if(showTime) {
         static int lastTime=0;
-        static char theTime[13];
+        static char theTime[13*3];
         float size =.15;
         struct tm* thisTime;
         time_t rawtime;
@@ -561,39 +578,39 @@ static void display_fps_subby()
         thisTime = localtime ( &rawtime );
 
         if(thisTime->tm_min != lastTime) {
-            char h[3];
-            char m[3];
-            char ampm[3] = " ";
+            char h[13];
+            char m[13];
+            char ampm[13] = " ";
 
             lastTime = thisTime->tm_min;
 
             if(thisTime->tm_min < 10) {
-                sprintf(m, "0%d", thisTime->tm_min);
+                snprintf( m, sizeof( m ), "0%d", thisTime->tm_min );
             } else {
-                sprintf(m, "%d", thisTime->tm_min);
+                snprintf( m, sizeof( m ), "%d", thisTime->tm_min );
             }
 
             if(show24hour) {
-                sprintf(h, "%d", thisTime->tm_hour);
+                snprintf( h, sizeof( h ), "%d", thisTime->tm_hour );
             } else {
                 int newhour;
 
                 if(thisTime->tm_hour > 12) {
                     newhour = thisTime->tm_hour-12;
-                    sprintf(ampm,"%s","PM");
+                    snprintf( ampm, sizeof( ampm ), "%s", "PM" );
                 } else {
                     newhour = thisTime->tm_hour;
                     if(newhour == 0) newhour = 12;
-                    sprintf(ampm,"%s","AM");
+                    snprintf( ampm, sizeof( ampm ), "%s", "AM" );
                 }
-                sprintf(h, "%d", newhour);
+                snprintf( h, sizeof( h ), "%d", newhour );
             }
 
-            sprintf(theTime, "%s:%s %s",h,m,ampm);
+            snprintf( theTime, sizeof( theTime ), "%s:%s %s", h, m, ampm );
         }
 
         // float timesize = subby_ScoreSize; // -((.15*timesize*(10-1.5))/2.0)
-        rTextField the_time(.7,.9,.15*size, .3*size);
+        rTextField the_time(.9-.2*rTextField::AspectWidthMultiplier(),.9,.15*size*rTextField::AspectWidthMultiplier(), .3*size);
 
         the_time << "0xffffff" << theTime;
     }
