@@ -129,7 +129,12 @@ public:
 class gBrowserMenuItem: public uMenuItem
 {
 protected:
+    bool displayHelp_;
+    REAL helpAlpha_;
+    
     gBrowserMenuItem(uMenu *M,const tOutput &help): uMenuItem( M, help )
+      , displayHelp_{false}
+      , helpAlpha_(0.0f)
     {
     }
 
@@ -137,6 +142,13 @@ protected:
     virtual bool Event( SDL_Event& event );
 
     virtual void RenderBackground();
+
+    virtual bool DisplayHelp( bool display, REAL y, REAL alpha )
+    {
+        helpAlpha_ = alpha;
+        displayHelp_ = display;
+        return false;
+    }
 };
 
 class gServerMenuItem: public gBrowserMenuItem
@@ -500,7 +512,8 @@ void gServerMenu::Render(REAL y,
 {
     if (sr_glOut)
     {
-        rTextField c(-.9f, y+text_height*.5, text_width, text_height);
+        REAL const xAspectMultiplier = rTextField::AspectWidthMultiplier();
+        rTextField c(-.9f * xAspectMultiplier, y+text_height*.5, text_width * xAspectMultiplier, text_height);
         c.SetWidth(1000);
 
         c.SetIndent(5);
@@ -517,7 +530,7 @@ void gServerMenu::Render(REAL y,
 
         if (ping.Len() > 1)
         {
-            text.SetPos(static_cast<int>(1.35/c.GetCWidth())  - tColoredString::RemoveColors( ping ).Len() - 1, true );
+            text.SetPos(static_cast<int>(1.35*xAspectMultiplier/c.GetCWidth())  - tColoredString::RemoveColors( ping ).Len() - 1, true );
             text << "0xRESETT";
             text << " " << ping;
         }
@@ -528,13 +541,13 @@ void gServerMenu::Render(REAL y,
 
         if (users.Len() > 1)
         {
-            text.SetPos(static_cast<int>(1.6/c.GetCWidth()) - tColoredString::RemoveColors( users ).Len(), false );
+            text.SetPos(static_cast<int>(1.6*xAspectMultiplier/c.GetCWidth()) - tColoredString::RemoveColors( users ).Len(), false );
             text << users;
         }
 
         if (score.Len() > 1)
         {
-            text.SetPos(static_cast<int>(1.8/c.GetCWidth()) - tColoredString::RemoveColors( score ).Len(), false );
+            text.SetPos(static_cast<int>(1.8*xAspectMultiplier/c.GetCWidth()) - tColoredString::RemoveColors( score ).Len(), false );
             text << score;
         }
 
@@ -683,17 +696,22 @@ void gServerMenuItem::Render(REAL x,REAL y,REAL alpha, bool selected)
 #endif
 }
 
+static REAL sg_menuBottom    = -.9;
+static REAL sg_requestBottom = -.9;
 
 void gServerMenuItem::RenderBackground()
 {
 #ifndef DEDICATED
+    REAL helpTopReal = sg_requestBottom*shrink + displace - .05;;
+
     gBrowserMenuItem::RenderBackground();
 
+    rTextField::SetDefaultColor( tColor(1,1,1) );
+
+    REAL const xAspectMultiplier = rTextField::AspectWidthMultiplier();
+    rTextField players( -.9, helpTopReal, text_width * xAspectMultiplier, text_height );
     if ( server )
     {
-        rTextField::SetDefaultColor( tColor(1,1,1) );
-
-        rTextField players( -.9, -.3, text_width, text_height );
         players << tOutput( "$network_master_players" );
         if ( server->UserNamesOneLine().Len() > 2 )
             players << server->UserNamesOneLine();
@@ -704,6 +722,27 @@ void gServerMenuItem::RenderBackground()
         uri << server->Url() << tColoredString::ColorString(1,1,1);
         players << tOutput( "$network_master_serverinfo", server->Release(), uri, server->Options() );
     }
+
+    {
+        players << "\n";
+        players.SetColor(tColor(1,1,1,displayHelp_ ? helpAlpha_ : 0));
+        players << Help();
+    }
+
+    REAL helpSpace = players.GetTop() - players.GetBottom();
+    REAL helpTop = -.85 + helpSpace;
+    REAL helpTopScaled = ( helpTop - displace )/shrink;
+    REAL helpTopMax = .25;
+    REAL helpTopMin = -.9;
+    if( helpTopScaled > helpTopMax )
+    {
+        helpTopScaled = helpTopMax;
+    }
+    if( helpTopScaled < helpTopMin )
+    {
+        helpTopScaled = helpTopMin;
+    }
+    sg_requestBottom = helpTopScaled;
 #endif
 }
 
@@ -808,6 +847,24 @@ bool gServerMenuItem::Event( SDL_Event& event )
 
 void gBrowserMenuItem::RenderBackground()
 {
+    if( menu )
+    {
+        double now = tSysTimeFloat();
+        static double lastTime = now;
+        if( sg_menuBottom > sg_requestBottom )
+        {
+            sg_menuBottom -= now - lastTime;
+        }
+        lastTime = now;
+        if( sg_menuBottom < sg_requestBottom )
+        {
+            sg_menuBottom = sg_requestBottom;
+        }
+
+        menu->SetBot( sg_menuBottom );
+        sg_requestBottom = -.9;
+    }
+
     sn_Receive();
     sn_SendPlanned();
 
